/*
 *  Copyright 2004 The Apache Software Foundation or its licensors, as
 *  applicable.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 *  implied.
 *
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package gcc.generator;

import java.util.Vector;

public class JBlockStatement extends JStatement
{
    protected Vector _localVars;
    protected Vector _statements;

    public JBlockStatement()
    {
        _localVars = new Vector();
        _statements = new Vector();
    }

    public boolean hasVariables()
    {
        return _localVars.size() > 0;
    }

    public boolean hasStatements()
    {
        return _statements.size() > 0;
    }

    public JLocalVariable newLocalVariable( Class type, String name )
    {
        return newLocalVariable( type, name, null );
    }

    public JLocalVariable newLocalVariable( Class type, String name, JExpression initExpr )
    {
        JLocalVariable v = new JLocalVariable( type, name );

        v.setInitExpression( initExpr );

        _localVars.add( v );

        return v;
    }

    public void deleteLocalVariable( JLocalVariable f )
    {
        _localVars.remove( f );
    }

    public Vector getLocalVariables()
    {
        return _localVars;
    }

    public void addStatement( JStatement s )
    {
        if (s != this)
        {
            _statements.add( s );
        }
    }

    public Vector getStatements()
    {
        return _statements;
    }
}
