/*
 *  Copyright 2004 The Apache Software Foundation or its licensors, as
 *  applicable.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 *  implied.
 *
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package gcc.generator;

import java.util.Vector;

public class JIfElseIfElseStatement extends JStatement
{
    protected JIfStatement       _ifStatement;
    protected Vector             _elseifStatements;
    protected JElseStatement     _elseStatement;

    public JIfElseIfElseStatement( JExpression if_expr )
    {
        _ifStatement = new JIfStatement( if_expr );
        _elseifStatements = new Vector();
        _elseStatement = new JElseStatement();
    }

    public void addIfStatement( JStatement s )
    {
        _ifStatement.addStatement( s );
    }

    public JIfStatement getIfStatement()
    {
        return _ifStatement;
    }

    public Vector getIfStatements()
    {
        return _ifStatement.getStatements();
    }

    public void addElseStatement( JStatement s )
    {
        _elseStatement.addStatement( s );
    }

    public JElseStatement getElseStatement()
    {
        return _elseStatement;
    }

    public Vector getElseStatements()
    {
        return _elseStatement.getStatements();
    }

    public JElseIfStatement getElseIf( JExpression e )
    {
        JElseIfStatement rc = null;
        int index = _elseifStatements.indexOf( e );

        if (index >= 0)
        {
            rc = (JElseIfStatement)_elseifStatements.get( index );
        }

        return rc;
    }

    public JElseIfStatement newElseIf( JExpression e )
    {
        JElseIfStatement rc = getElseIf( e );

        if (rc == null)
        {
            rc = new JElseIfStatement( e );
            _elseifStatements.add(rc);
        }

        return rc;
    }

    public void addCatchStatement( JExpression e, JStatement s )
    {
        JElseIfStatement eis = getElseIf( e );

        if (eis == null)
        {
            eis = newElseIf( e );
        }

        eis.addStatement( s );
    }

    public Vector getElseIfs()
    {
        return _elseifStatements;
    }
}
